#!/usr/bin/perl
=pod
#Daniel Jeffares 2017 
#dj@katipo.org
#gets the Hermes positions from the sam file
#will first make a bam, then sort it
#and get some stats,

#then locate the positions
#flag 99 reads:
samtools view -f 99 -q 30 test.sort.sam | cut -f 3,4 | perl -ane 'chomp;print join("\t",@F,"+"),"\n"'
#flag 83 reads:
samtools view -f 83 -q 30 test.sort.sam  | sam-rightmost-read-end.pl

=cut
use warnings;
use strict;


my ($prefix,$genome) = @ARGV;
unless ((@ARGV==2) and (-e "$prefix.sam")){die "USE: $0 samfileprefix genome > outputfile.txt\n\n";};


#make a bam
my $cmd = "samtools view -bt $genome $prefix.sam > $prefix.bam";
system "$cmd";
warn "running [$cmd]\n";


#sort it
$cmd = "samtools sort $prefix.bam $prefix.sort";
system "$cmd";
warn "running [$cmd]\n";

#index it
$cmd =  "samtools index $prefix.sort.bam";
system "$cmd";
warn "running [$cmd]\n";

#make some stats
$cmd = "stats --outfile $prefix.stats --ifile $prefix.ins.txt $prefix.sort.bam > stat.temp.$$";
system "$cmd";
warn "running [$cmd]\n";

#and now get the positions
#print a header
my $now = localtime;
print "#script $0 $prefix $genome
#date $now\nchr\tpos\tstrand\n";

#flag 99 reads
warn "parsing the flag 99 (fwd) reads\n";
open STATUS, "samtools view -f 99 -q 30 $prefix.sort.bam | cut -f 3,4 |" or die "Fek! cant open fork $!";
while (<STATUS>){
	chomp;
	my @a = split(" ",$_);
	print join("\t",@a,"+"),"\n";
}
close STATUS;

#flag 83 reads:
warn "parsing the flag 83 (fwd) reads\n";
open STATUS2, "samtools view -f 83 -q 30 $prefix.sort.bam |" or die "Fek! cant open fork $!";
while (<STATUS2>){
	chomp;
	my @t = split("\t");
	$_ = $t[5];
	my $end = $t[3]-1;
	s/(\d+)[NMD]/$end+=$1/eg;
	print join ("\t",$t[2],$end,"-"), "\n";
}

#clean up
warn "cleaning up\n";
system "rm -f $prefix.sam";
system "rm -f $prefix.bam";
system "rm -f stat.temp.$$";

warn "Done [$prefix,$genome]\n";
exit;
