#!/share/apps/perl/bin/perl

=pod
#Daniel Jeffares 2017 
#dj@katipo.org

check that read1 has unique sequnce at 5' $nmer length
put to non-unique file if not & count

reads are:

#76 nt reads
16mer?		10ntbar	7nntHDS	17ntHermesRC			26nt genome
GCAGCTTTTAATGCAA TATAGCCTAC CCTGTGG CTTGTTGAAGTTCTCTG CTAGTGATGTATGACGCAGATGGTTA


=cut

use strict;
use warnings;
use Bio::SeqIO;
use Data::Dumper;
use Benchmark;

my ($fastq1, $nmer1, $nmer2, $last,$record_base_comp) = @ARGV;
unless (@ARGV >= 2 and -e $fastq1){
die "\nuse:$0 fastq1 nmer1 nmer2 [last read to process(all)] [record_base_comp(1/0)]\n
nmers should be start-end (1-based)
NB: This version leaves the re-pairing to tally.\n\n"
};
$last ||= 9e999;
$record_base_comp ||= 0;
my $basecomp = {};


#check fastq1
#read positions that should be thrown out
my %positions=();
my $totalremoved;
my $seen={};
#$seen->{16mer}{26mere_at_end}=1;


#parse nmers
my ($nmer1_start,$nmer1_end) = split(/-/,$nmer1);
my ($nmer2_start,$nmer2_end) = split(/-/,$nmer2);

#time it
my $time0 = new Benchmark;

#gunzip, if required
if ($fastq1 =~ /\.gz/){
	warn "gunzipping $fastq1";
	system "gunzip $fastq1";
	$fastq1 =~ s/\.gz//g;
}

#get the non-unique from read1
print "##Processing Fastq1 file:$fastq1\n";
my $in = Bio::SeqIO->new(-format => 'fastq',	-file => "$fastq1");
my $outOK = Bio::SeqIO->new(-format => 'fastq',	-file => ">unique.$fastq1");
my $outredundant = Bio::SeqIO->new(-format => 'fastq',	-file => ">redundant.$fastq1");
my $counter=0;
while (my $seq = $in->next_seq){
	my $sub = $seq->subseq($nmer1_start,$nmer1_end);
	
	if ($record_base_comp){addcounts($sub)};
	
	
	my $sub2 = $seq->subseq($nmer2_start,$nmer2_end);
	my $id = $seq->id();
	
	if ($seen->{$sub}{$sub2}){
			$outredundant->write_seq($seq);
		 	$positions{$counter}=$id;
		 	$totalremoved++;
	}
	else{
		$outOK->write_seq($seq);
	}
	$seen->{$sub}{$sub2}++;
	
	unless ($counter % 10000){
		warn "##Parsed:$counter Duplicates:" , scalar keys %positions , "\n";
		print "##Parsed:$counter Duplicates:" , scalar keys %positions , "\n";
	};

	$counter++;
	if ($counter==$last){last};
}


#collect date and print a short report:
my $date = localtime;

my $time1 = new Benchmark;
my $timdiff = timediff($time1, $time0);
my $time = timestr($timdiff);

print "#script $0
#date $date 
#files: $fastq1
#nmers: $nmer1 $nmer2

#total_duplicates_removed: $totalremoved
#total_reads $counter
#time_to_run $time 
nmer\tgenomeseq\tcount\n";

for my $s1 (sort keys %{$seen}){
	for my $s2 (sort keys %{$seen->{$s1}}){
		if ($seen->{$s1}{$s2} > 1){
			print "$s1\t$s2\t$seen->{$s1}{$s2}\n";
		}
	}
}
 
 

if ($record_base_comp){
	open (B, ">$fastq1.nmer-base-compostion.txt");
	print B "#random nmer bias\n";
	print B  join("\t",qw(position A G C T N)),"\n";
	for my $i (sort keys %{$basecomp}){
		my $pos = $i+1;
		print B "$pos\t";
		for my $j (qw(A G C T N)){
			$basecomp->{$i}{$j}||= 0;
			print B "$basecomp->{$i}{$j}\t";
		}
		print B "\n";
	}
	close B;
	warn "\n\nNmer base compostion data is in: $fastq1.nmer-base-compostion.txt\n\n";
} 


#subroutine
sub addcounts{
	#counts the AGCT at eash postion in the given Nmer
	my ($in) = @_;
	my @letters = split(//,$in);
	for my $i (0..scalar(@letters)-1){
		$basecomp->{$i}{$letters[$i]}++;
	}

}

exit;
